'''scicm is a small package containing several colour maps, aimed at filling gaps in the
availability of good (i.e. perceptually linear) in both matplotlib and in other available
packages (like cmocean and CMasher). All colour maps in this package have been created
using viscm.'''
from . import cm
from . import tools
